﻿using System;
namespace SpikingNeuron
{
  class SpikingNeuronProgram
  {
    static void Main(string[] args)
    {
      Console.WriteLine("\nBegin spiking neuron demo\n");

      int[][] inputs = new int[3][];
      inputs[0] = new int[] { 0, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1 };
      inputs[1] = new int[] { 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 1, 1, 0, 1, 1 };
      inputs[2] = new int[] { 1, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 1, 1, 1, 1, 1 };
      // t =                  0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15
 
      Console.WriteLine("The inputs are: \n");
      for (int i = 0; i < inputs.Length; ++i)
        ShowVector(inputs[i], false);
      Console.WriteLine("");

      int[] output = new int[16]; // process 16 input values

      int[] wts = new int[] { 4, -2, 3 };
      Console.Write("The weights are: ");
      ShowVector(wts, true);
      Console.WriteLine("");

      int leak = 1;
      Console.WriteLine("The leak potential is:           " + leak);
 
      int v = 0; // electrical potential (voltage)
      int thresh = 8; // potential value needed to fire an output spike
      int spike = 4;  // increase in v at spike event
      int tNext = 0; // next time when neuron is active
      int latency = 2; // number t inactive after spike

      Console.WriteLine("The threshold potential is:      " + thresh);
      Console.WriteLine("The spike potential is:          " + spike);
      Console.WriteLine("The post-spike latency time is:  " + latency);
      Console.WriteLine("\nStarting processing\n");

      for (int t = 0; t < 16; ++t)
      {
        Console.WriteLine("---------------------------------------------------");
        Console.Write(" ");
        Console.Write("t = ");
        if (t <= 9) Console.Write(" ");
        Console.Write(t + ". ");

        Console.Write("Inputs = " + inputs[0][t] + " " + inputs[1][t] + " " + inputs[2][t]);

        if (t != tNext) // neuron is not active
        {
          Console.Write(". Neuron is inactive. ");
          Console.WriteLine("V = " + v);
          output[t] = 0;
        }
        else // neuron is active
        {
          Console.Write(". Neuron is   active. ");
         
          int sum = 0;
          for (int j = 0; j < inputs.Length; ++j)
            sum += inputs[j][t] * wts[j];
          v = v + sum;

          v = v - leak;
          if (v < 0)
            v = 0;

          Console.WriteLine("V = " + v);

          if (v >= thresh) // spike and reset
          {
            v = v + spike;
            Console.WriteLine(" Spiking, V = " + v);
            output[t] = 1;
            v = 0;
            tNext = t + 1 + latency;
          }
          else
          {
            output[t] = 0;
            tNext = t + 1;
          }

        } // active
        
      } // t
      Console.WriteLine("---------------------------------------------------\n");
      Console.WriteLine("Output spike train = ");
      ShowVector(output, false);

      Console.WriteLine("\nEnd spiking neuron demo\n");
      Console.ReadLine();
    } // Main

    static void ShowVector(int[] vector, bool plus)
    {
      for (int i = 0; i < vector.Length; ++i)
      {
        if (plus == true && vector[i] >= 0)
          Console.Write("+");
        Console.Write(vector[i] + " ");
      }
      Console.WriteLine("");
    }

  } // Program

} // ns
